#!/usr/bin/perl

# This is a little program to enable you to run the "HP Scanning" tool from
# the HP Solution Center, without running the HP Solution Center, since HP has
# shamefully desupported the HP Solution Center, and left in the lurch all the
# owners of old printers # which rely on it. Their "HP Scan and Capture" tool,
# which they offer as a substitute, is extrenekt inferior, because it can't
# make OCR'd pdf documents.
#
# This program does two things:
#
# 1. It looks up the needed information to enable it to start the HP Scanning
# tool, and does so.
#
# 2. It leaves behind a little file canned no_solution_center_cmd.bat, which
# will directly start the HP Scanning tool for your printer.
#
# by Dave Burton   daveburton.wordpress.com   Feb. 24, 2021
#
#
# This program is uncopyrighted, but, as a courtesy, I ask that if you copy or
# modify it, that you please retain this notice.
#
# This Perl program is compatible both with "normal" Perl 5 under Windows,
# and also with the tiny standalone perl4.exe for Win32.
#
# This program was made possible by the expert sluthing of "Wstack":
# https://h30434.www3.hp.com/t5/Scanning-Faxing-Copying/HP-Solution-Center-does-not-work/m-p/7933933/highlight/true#M240591


$| = 1;  # immediate output of debug prints

# What version of Perl are we running?
$perlver = "3 or earlier";
if ($] =~ /\$\$Revision\:\s*([0-9.]+)\s/) {
   $perlver = $1;  # probably 4.something
} elsif ($] =~ /([0-9][0-9.]*)/) {
   $perlver = $1;  # probably 5.something or 6.something
}
print "You are using Perl version $perlver\n";

# Find the path of the folder containing this program:
$programpath = $0;
print "$programpath\n";
$programfolder = '';
if ($programpath =~ /(^.*[\:\\])[^\:\\]*$/) {
   $programfolder = $1;
}
# print "programfolder = '$programfolder'\n";

print "Querying the registry for a list of printers..";

# this is an example of what we expect to find:
# HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Print\Printers\HP Officejet 6500 E709n Series

$cmd1 = 'REG QUERY "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Print\Printers"';
$result1 = `$cmd1`;
print ".\n";
@results = split(/\r\n|\n/, $result1);
@printerkeys = ();
foreach $x (@results) {
   if ($x =~ /^HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Print\\Printers\\(.*HP.*)$/) {
      $printername = $1;
      # print "-----\n$x\n$printername\n";
      push(@printerkeys, $x);
   }
}
#print "-----\n";
$num_hp_printer_entries = $#printerkeys+1;
print "Found $num_hp_printer_entries HP printer entries in the registry.\n";

print "Querying the registry for SoftwareCUEContextID entries...\n";
%foundit = ();
foreach $x (@printerkeys) {
   $x =~ /^HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Print\\Printers\\(.*HP.*)$/;
   $printername = $1;
   $key = $x . "\\PrinterDriverData";
   $cmd2 = 'REG QUERY "' . $key . '" /v SoftwareCUEContextID 2>nul';
   $result2 = `$cmd2`;
   $contextid = '';
   @lns = split( /\r\n|\n/, $result2 );
   for $ln (@lns) {
      if ($ln =~ /^ *SoftwareCUEContextID *REG_SZ *(.*)$/) {
         # example: "#Hewlett-Packard#HP Officejet 6500 E709n Series#1476669342"
         $contextid = $1;
      }
   }
   if ($contextid) {
      if ($foundit{$contextid}) {
         $printername = $foundit{$contextid} . "\n" . $printername;
      }
      $foundit{$contextid} = $printername;
   }
}

# print "=-=-=-=-=-=-=-=-=\n";
$num_entries = keys %foundit;
print "Found $num_entries HP printer SoftwareCUEContextID entries...\n";
$last_contextid = '';

for $contextid (keys %foundit) {
   $last_contextid = $contextid;
   $printername = $foundit{$contextid};
   $printername =~ s/\n/\n  /g;
   print "printer name(s):\n  $printername\ncontextID = '$contextid'\n";
}

# in case of a 32-bit version of Windows, or D: Windows system drive...
$progx86path = $ENV{'ProgramFiles(x86)'};
if (!$progx86path) {
   # just in case the environment variable lookup didn't work
   $progx86path = 'C:\Program Files (x86)';
}
$HP_scanning_dot_exe = $progx86path . '\HP\Digital Imaging\bin\hpqkygrp.exe';

if ('' ne $last_contextid) {
   $args = '-n"' . $last_contextid . '" -aSCAN -fSCAN_DOCUMENT';

   # With Perl 5 we could directly execute the HP Scanning tool (but it doesn't work in Perl 4):
   # $cmd3 = '"C:\Program Files (x86)\HP\Digital Imaging\bin\hpqkygrp.exe" -n"' . $last_contextid . '" -aSCAN -fSCAN_DOCUMENT';
   # $cmd3 = '"' . $HP_scanning_dot_exe . '" ' . $args;
   # print "Starting the HP Scanning tool; the command-line is:\n$cmd3\n";
   # `$cmd3`;
   # exit(0);

   # For compatibility with Perl 4, we create "no_solution_center_cmd.bat" in
   # the same folder with this program, which contains the needed command.
   # Then we'll run it.

   $generated_scriptfile = $programfolder . "no_solution_center_cmd.bat";

   print "\n*** CREATING: \"$generated_scriptfile\" ***\n";
   open(CMDFILE, ">$generated_scriptfile");
   print CMDFILE "REM Run the HP Scanning tool without HP Solution Center\n";

   # For user convenience, list all possible SoftwareCUEContextID values in
   # comments in the generated .bat file:
   for $contextid (keys %foundit) {
      print CMDFILE 'REM "' . $contextid . "\"\n";
   }
   print CMDFILE "\n\"$HP_scanning_dot_exe\" $args\n";
   close(CMDFILE);
   print "\n";

   # Now we run the generated .bat file, to run the HP Scanning tool.
   system( $generated_scriptfile );
   print "Back from running '$generated_scriptfile'\n";

   if ($num_entries != 1) {
      print "Warning: more than one HP printer with a SoftwareCUEContextID entry was found.\nThe last one was used.\n";
      print "Press ENTER to finish the script:";
      <STDIN>;
   }
} else {
   print "ERROR: no HP printers with SoftwareCUEContextID entries were found in the registry.\n";
   print "Press ENTER to quit the script:";
   <STDIN>;
}

__END__

